from distutils.ccompiler import get_default_compiler, new_compiler
from distutils.extension import Extension
from distutils.command.build_ext import build_ext
from distutils.command.sdist import sdist
import shutil
from glob import glob
from os.path import basename, join as pjoin
from subprocess import Popen, PIPE
import platform
import re
import stat
from ctypes import *

from .msg import *
from .fetch import fetch_libczmq

HERE = os.path.dirname(__file__)
ROOT = os.path.dirname(HERE)


def stage_platform_header(zmqroot):
    platform_h = pjoin(zmqroot, 'src', 'platform.h')

    if os.path.exists(platform_h):
        info("already have platform.h")
        return

    if os.name == 'nt':
        # stage msvc platform header
        platform_dir = pjoin(zmqroot, 'builds', 'msvc')
    else:
        if not os.path.exists(pjoin(zmqroot, 'configure')):
            info('attempting bash autogen.sh')
            p = Popen('./autogen.sh', cwd=zmqroot, shell=True, stdout=PIPE, stderr=PIPE)
            o, e = p.communicate()
            if p.returncode:
                raise RuntimeError('Failed to run autoconf...')

        configure_cmd = [
            'libzmq_CFLAGS=-I../zeromq/include',
            'libzmq_LIBS=-L../zeromq/src',
            './configure'

        ]
        info("attempting ./configure to generate platform.h with: {}".format(configure_cmd))
        p = Popen(' '.join(configure_cmd), cwd=zmqroot, shell=True, stdout=PIPE, stderr=PIPE)

        o, e = p.communicate()
        if p.returncode:
            warn("failed to configure libczmq:\n%s" % e)

            if sys.platform == 'darwin':
                platform_dir = pjoin(HERE, 'include_darwin')
            elif sys.platform.startswith('freebsd'):
                platform_dir = pjoin(HERE, 'include_freebsd')
            elif sys.platform.startswith('linux-armv'):
                platform_dir = pjoin(HERE, 'include_linux-armv')
            else:
                platform_dir = pjoin(HERE, 'include_linux')
        else:
            return

    info("staging platform.h from: %s" % platform_dir)
    shutil.copy(pjoin(platform_dir, 'platform.h'), platform_h)


class ConfigureSDist(sdist):
    def run(self):
        bundledir = "bundled"

        line()
        info("Using bundled libczmq")

        # fetch sources for libzmq extension:
        if not os.path.exists(bundledir):
            os.makedirs(bundledir)

        fetch_libczmq(bundledir)

        bundledincludedir = 'czmq'

        if not os.path.exists(bundledincludedir):
            os.makedirs(bundledincludedir)

        files = [
            pjoin(bundledir, 'czmq', 'bindings', 'python', 'czmq', '_czmq_ctypes.py'),
            pjoin(bundledir, 'czmq', 'bindings', 'python', 'czmq', '__init__.py')
        ]

        for f in files:
            shutil.copyfile(f, pjoin(bundledincludedir, basename(f)))


class Configure(build_ext):

    user_options = build_ext.user_options + [
        ('czmq=', None, "libczmq install prefix"),
        ('build-base=', 'b', "base directory for build library"),  # build_base from build
    ]

    def initialize_options(self):
        build_ext.initialize_options(self)
        self.build_base = 'build'

    # DON'T REMOVE: distutils demands these be here even if they do nothing.
    def finalize_options(self):
        build_ext.finalize_options(self)
        self.tempdir = pjoin(self.build_temp, 'scratch')
        self.has_run = False

    @property
    def compiler_type(self):
        compiler = self.compiler
        if compiler is None:
            return get_default_compiler()
        elif isinstance(compiler, str):
            return compiler
        else:
            return compiler.compiler_type

    @property
    def cross_compiling(self):
        return self.config['bdist_egg'].get('plat-name', sys.platform) != sys.platform

    def bundle_libczmq_extension(self):
        bundledir = "bundled"
        ext_modules = self.distribution.ext_modules

        if ext_modules and any(m.name == 'czmq.libczmq' for m in ext_modules):
            # I've already been run
            return

        line()
        info("Using bundled libczmq")

        # fetch sources for libzmq extension:
        if not os.path.exists(bundledir):
            os.makedirs(bundledir)

        fetch_libczmq(bundledir)

        stage_platform_header(pjoin(bundledir, 'czmq'))

        czmq_sources = []
        for f in glob(pjoin(bundledir, 'czmq', 'src', '*.c')):
            if f not in ['bundled/czmq/src/czmq_selftest.c', 'bundled/czmq/src/zsp.c'] and 'test_' not in f:
                czmq_sources.append(f)

        czmq_sources.append(pjoin('buildutils', 'czmq', 'initlibczmq.c'))

        compile_args = [
            '-std=c99',
            '-Wno-strict-prototypes',
            '-Wno-unused-variable',
            '-Wno-unused-function',
        ]

        if sys.platform != 'darwin':
            compile_args.append('-Wno-unused-but-set-variable')

        library_dirs = [
            pjoin(self.build_lib, 'czmq'),
        ]

        if os.name != 'nt':
            library_dirs.append('/usr/local/lib')  # osx needs this to find ossp-uuid

        libraries = ['zmq', 'uuid']

        if re.search(r'centos-7|Ubuntu-16', platform.platform()):
            libraries.append('systemd')

        libczmq = Extension(
            'czmq.libczmq',
            sources=czmq_sources,
            include_dirs=[
                pjoin(bundledir, 'czmq', 'include'),
                pjoin(bundledir, 'zeromq', 'include')
            ],
            libraries=libraries,
            library_dirs=library_dirs,

            extra_compile_args=compile_args,
            # http://stackoverflow.com/a/19147134
            runtime_library_dirs=['zmq', 'czmq', '.'],
        )

        # http://stackoverflow.com/a/32765319/7205341
        if sys.platform == 'darwin':
            from distutils import sysconfig
            vars = sysconfig.get_config_vars()
            vars['LDSHARED'] = vars['LDSHARED'].replace('-bundle', '-dynamiclib')

        # register the extension:
        self.distribution.ext_modules.insert(1, libczmq)

        libczmq.include_dirs.append(bundledir)

        libczmq.define_macros.append(('WITH_MAKECERT', 0))
        libczmq.define_macros.append(('WITH_TEST_ZGOSSIP', 0))
        libczmq.define_macros.append(('CZMQ_BUILD_DRAFT_API', 1))

        cc = new_compiler(compiler=self.compiler_type)
        cc.output_dir = self.build_temp

        bundledincludedir = 'czmq'

        if not os.path.exists(bundledincludedir):
            os.makedirs(bundledincludedir)

        if not os.path.exists(pjoin(self.build_lib, bundledincludedir)):
            os.makedirs(pjoin(self.build_lib, bundledincludedir))

        files = [
            pjoin(bundledir, 'czmq', 'bindings', 'python', 'czmq', '_czmq_ctypes.py'),
            pjoin(bundledir, 'czmq', 'bindings', 'python', 'czmq', '__init__.py')
        ]

        for f in files:
            shutil.copyfile(f, pjoin(bundledincludedir, basename(f)))
            shutil.copyfile(f, pjoin(self.build_lib, bundledincludedir, basename(f)))

    def run(self):

        self.bundle_libczmq_extension()


import pytest
from pyzyre.utils import resolve_endpoint, address_to_interface, resolve_address
import sys
import netifaces as ni


@pytest.fixture
def iface():
    if sys.platform == 'darwin':
        return 'en0'
    else:
        if 'eth0' in ni.interfaces():
            return 'eth0'
        else:
            return ni.interfaces()[1]


def test_resolve_address(iface):
    addr = resolve_address(iface)
    assert addr


def test_resolve_interface(iface):
    addr = resolve_address(iface)
    assert addr

    i = address_to_interface(addr)
    assert i


def test_resolve_endpoint():
    e = resolve_endpoint('*', port=5432)
    assert e



#!/usr/bin/env python

import os
import zmq.auth
from argparse import ArgumentParser
from pprint import pprint

CERTS_PATH = os.getenv('ZYRE_CERTS_PATH', os.path.expanduser('~/.curve'))


def main():
    if zmq.zmq_version_info() < (4,0):
        raise RuntimeError("Security is not supported in libzmq version < 4.0. libzmq version {0}".format(
            zmq.zmq_version()))

    p = ArgumentParser(add_help=False)
    p.add_argument('-d', '--debug', action="store_true")
    p.add_argument('-v', '--verbose', action="store_true")
    p.add_argument('--path', help='specify base path [default %(default)s]', default=CERTS_PATH)
    p.add_argument('--name', help='specify key name [default %(default)s]', default='zyre')

    args = p.parse_args()

    certs = zmq.auth.create_certificates(args.path, 'test', metadata={'name': args.name})
    for c in certs:
        os.chmod(c, 0600)

    print("Generated certs in: %s" % CERTS_PATH)


if __name__ == '__main__':
    main()



import logging
import sys
import textwrap
from argparse import ArgumentParser, RawDescriptionHelpFormatter

import zmq
from pyzyre.client import Client
from zmq.eventloop import ioloop
from pyzyre.utils import get_argument_parser, setup_logging, setup_curve

logger = logging.getLogger('pyzyre.chat')


def main():
    p = get_argument_parser()
    p = ArgumentParser(
        description=textwrap.dedent('''\
            example usage:
                $ zyre-chat -d
            '''),
        formatter_class=RawDescriptionHelpFormatter,
        prog='zyre-chat',
        parents=[p]
    )

    args = p.parse_args()

    setup_logging(args)

    args.cert = setup_curve(args)

    ioloop.install()
    loop = ioloop.IOLoop.instance()

    client = Client(loop=loop, **args.__dict__)

    def on_stdin(s, e):
        content = s.readline().rstrip()

        if content.startswith('CLIENT:'):
            address, message = content.split(' - ')
            address = address.split(':')[1]
            client.whisper(message, address)
        else:
            client.shout(args.group, content.encode('utf-8'))

    client.start_zyre()

    loop.add_handler(client.actor, client.handle_message, zmq.POLLIN)
    loop.add_handler(sys.stdin, on_stdin, ioloop.IOLoop.READ)

    try:
        loop.start()
    except KeyboardInterrupt:
        logger.info('SIGINT Received')

    logger.info('shutting down..')

    client.stop_zyre()


if __name__ == '__main__':
    main()



import logging
import textwrap
from argparse import ArgumentParser, RawDescriptionHelpFormatter

import zmq
from .client import Client, DefaultHandler
from zmq.eventloop import ioloop
from .utils import setup_logging, get_argument_parser, setup_curve

logger = logging.getLogger(__name__)


class GatewayHandler(DefaultHandler):
    def __init__(self, pub):
        self.pub = pub

    def on_shout(self, client, group, peer, address, message):
        self.pub.send_multipart([group, message])


# see examples/pub.py and sub.py
def main():
    p = get_argument_parser()
    p = ArgumentParser(
        description=textwrap.dedent('''\
                example usage:
                    $ zyre-gateway -d
                '''),
        formatter_class=RawDescriptionHelpFormatter,
        prog='zyre-gateway',
        parents=[p]
    )

    p.add_argument('-p', '--pub', help='endpoint to bind for PUB socket [default %(default)s]',
                   default="tcp://*:5001")
    p.add_argument('-u', '--pull', help='endpoint to bind for PULL socket [default %(default)s]',
                   default="tcp://*:5002")

    args = p.parse_args()

    setup_logging(args)

    args.cert = setup_curve(args)

    ioloop.install()
    loop = ioloop.IOLoop.instance()

    context = zmq.Context()
    pub = context.socket(zmq.PUB)
    if not args.pub.startswith(('tcp://', 'udp://', 'ipc://')):
        args.pub = 'tcp://%s' % args.pub

    logger.info('listing for PUB on %s' % args.pub)
    pub.bind(args.pub)

    pull = context.socket(zmq.PULL)
    if not args.pull.startswith(('tcp://', 'udp://', 'ipc://')):
        args.pull = 'tcp://%s' % args.pull

    logger.info('listing for PULL on %s' % args.pull)
    pull.bind(args.pull)

    client = Client(
        handler=GatewayHandler(pub),
        loop=loop,
        **args.__dict__
    )

    def handle_pull(*args):
        req = pull.recv_multipart()
        cmd = req[0]
        args = req[1:]
        if cmd == 'PUB':
            client.shout(*args)
            pub.send_multipart(args)
        elif cmd == 'SUB':
            client.join(*args)

    client.start_zyre()
    loop.add_handler(client.actor, client.handle_message, zmq.POLLIN)
    loop.add_handler(pull, handle_pull, zmq.POLLIN)

    terminated = False
    while not terminated:
        try:
            logger.info('starting loop...')
            loop.start()
        except KeyboardInterrupt:
            logger.info('SIGINT Received')

            terminated = True
        except Exception as e:
            terminated = True
            logger.error(e)

    logger.info('shutting down..')

    client.stop_zyre()


if __name__ == '__main__':
    main()



import os.path
from argparse import ArgumentParser
import logging
from .znetwork import *

from czmq import Zcert

from pyzyre.constants import VERSION, ENDPOINT, PUBLIC_KEY, GOSSIP_PUBLIC_KEY, SECRET_KEY, CURVE_ALLOW_ANY, ZYRE_GROUP, \
    NODE_NAME, CERT_PATH, LOG_FORMAT, LOGLEVEL, GOSSIP_BIND, GOSSIP_CONNECT

if not os.path.exists(CERT_PATH):
    CERT_PATH = None


logger = logging.getLogger(__name__)


def get_argument_parser(advanced=True):
    BasicArgs = ArgumentParser(add_help=False)
    BasicArgs.add_argument('-d', '--debug', action="store_true")
    BasicArgs.add_argument('-v', '--verbose', action="store_true")
    BasicArgs.add_argument('-V', '--version', action="version", version=VERSION)

    if not advanced:
        return ArgumentParser(parents=[BasicArgs], add_help=False)

    BasicArgs.add_argument('-i', '--interface', help='specify zsys_interface for beacon')
    BasicArgs.add_argument('-l', '--endpoint', help='specify ip listening endpoint [default %(default)s]',
                           default=ENDPOINT)
    BasicArgs.add_argument('--advertised-endpoint', help='[GOSSIP ONLY] advertise an alternative endpoint (NAT)')

    BasicArgs.add_argument('--name', help='specify node name [default %(default)s]', default=NODE_NAME)
    BasicArgs.add_argument('--group', default=ZYRE_GROUP)

    BasicArgs.add_argument('--cert', help="specify local cert path")
    BasicArgs.add_argument('--curve', help="enable CURVE (TLS)", action="store_true")
    BasicArgs.add_argument('--publickey', help="specify CURVE public key [default %(default)s]", default=PUBLIC_KEY)
    BasicArgs.add_argument('--secretkey', help="specify CURVE secret key [default %(default)s]", default=SECRET_KEY)

    BasicArgs.add_argument('--gossip-bind', help='bind gossip endpoint on this node [default %(default)s]',
                           default=GOSSIP_BIND)
    BasicArgs.add_argument('--gossip-connect', help='connect to gossip endpoint [default %(default)s]',
                           default=GOSSIP_CONNECT)

    BasicArgs.add_argument('--gossip-cert', help="specify gossip cert path [default %(default)s]", default=CERT_PATH)
    BasicArgs.add_argument('--gossip-publickey', help='specify CURVE public key [default %(default)s]',
                           default=GOSSIP_PUBLIC_KEY)
    BasicArgs.add_argument('--zauth-curve-allow', help="specify zauth curve allow [default %(default)s]",
                           default=CURVE_ALLOW_ANY)

    return ArgumentParser(parents=[BasicArgs], add_help=False)


def setup_logging(args):
    loglevel = logging.getLevelName(LOGLEVEL)

    if args.verbose:
        loglevel = logging.INFO

    if args.debug:
        loglevel = logging.DEBUG

    console = logging.StreamHandler()
    logging.getLogger('').setLevel(loglevel)
    console.setFormatter(logging.Formatter(LOG_FORMAT))
    logging.getLogger('').addHandler(console)


def setup_curve(args):
    if not args.curve and not args.publickey and not args.cert and not args.gossip_publickey and not args.gossip_cert:
        return None

    if args.cert:
        cert = Zcert.load(args.cert)
        logger.info('CURVE configured via CERT...')
        return cert

    if args.curve or args.publickey or args.gossip_publickey:
        cert = Zcert()
        if args.publickey:
            if not args.secretkey:
                raise SystemExit

            cert = Zcert.new_from_txt(args.publickey, args.secretkey)

    if args.gossip_cert:
        gcert = Zcert.load(args.gossip_cert)
        args.gossip_publickey = gcert.public_txt()
        if not args.gossip_connect:
            args.gossip_connect = (gcert.meta('gossip-endpoint'))

        if not cert:
            cert = Zcert()

    logger.info('CURVE configured...')
    return cert






import logging
from argparse import ArgumentParser, RawDescriptionHelpFormatter
import textwrap
import select
import sys

import zmq
from pyzyre.constants import PYVERSION, ZMQ_LINGER, ZYRE_GATEWAY
from pyzyre.utils import setup_logging, get_argument_parser


def main():
    p = get_argument_parser(advanced=False)
    p = ArgumentParser(
        description=textwrap.dedent('''\
                    example usage:
                        $ cat test.eml | zyre-pipe --gateway tcp://192.168.1.2:49100 [-v|-d]
                    '''),
        formatter_class=RawDescriptionHelpFormatter,
        prog='zyre-pipe',
        parents=[p]
    )

    p.add_argument('--gateway', help="sepcify the gateway address [default %(default)s]", default=ZYRE_GATEWAY)
    p.add_argument('--linger', help="specify zmq_linger [default %(default)s]", default=ZMQ_LINGER)

    args = p.parse_args()

    logger = logging.getLogger('')
    setup_logging(args)

    if not select.select([sys.stdin, ], [], [], 0.0)[0]:
        logger.info('Nothing in STDIN to send..')
        raise SystemExit

    ctx = zmq.Context()
    s = ctx.socket(zmq.PUSH)
    s.setsockopt(zmq.LINGER, args.linger)
    s.connect(args.address)

    content = sys.stdin.read().strip('\n')

    if PYVERSION == 2:
        content = unicode(content, encoding='utf-8', errors='ignore')

    logger.info('sending..')
    s.send_multipart([content.encode('utf-8')])
    logger.info('sent...')
    s.close()

    logger.info('done')


if __name__ == '__main__':
    main()



from ._version import get_versions
VERSION = get_versions()['version']
del get_versions
import os
import sys

PYVERSION = 2
if sys.version_info > (3,):
    PYVERSION = 3


ZYRE_GROUP = os.environ.get('ZYRE_GROUP', 'ZYRE')
SERVICE_PORT = os.environ.get('ZYRE_SERVICE_PORT', '49155')

GOSSIP_PORT = os.environ.get('ZYRE_SERVICE_PORT', '49154')
GOSSIP_CONNECT = os.getenv('ZYRE_GOSSIP_CONNECT')
ENDPOINT = os.getenv('ZYRE_ENDPOINT')
GOSSIP_BIND = os.getenv('ZYRE_GOSSIP_BIND')

LOG_FORMAT = '%(asctime)s - %(levelname)s - %(name)s[%(lineno)s] - %(message)s'

CURVE_ALLOW_ANY = "*"

PUBLIC_KEY = os.getenv('ZYRE_PUBLIC_KEY')
SECRET_KEY = os.getenv('ZYRE_SECRET_KEY')
GOSSIP_PUBLIC_KEY = os.getenv('ZYRE_GOSSIP_PUBLIC_KEY')

NODE_NAME = os.getenv('ZYRE_NODE_NAME')
ZAUTH_TRACE = os.getenv('ZAUTH_TRACE', False)

CERT_PATH = os.getenv('ZYRE_CERT_PATH', '~/.certs')

LOGLEVEL = os.getenv('ZYRE_LOGLEVEL', 'ERROR')

ZMQ_LINGER = os.getenv('ZMQ_LINGER', 0)
ZYRE_GATEWAY = os.getenv('ZYRE_GATEWAY')


class DefaultHandler(object):
    def on_shout(self, client, group, peer, address, message):
        pass

    def on_whisper(self, client, peer, message):
        pass

    def on_enter(self, client, peer):
        pass

    def on_join(self, client, peer, group):
        pass

    def on_leave(self, client, peer, group):
        pass

    def on_evasive(self, client, peer):
        pass

    def on_exit(self, client, peer):
        pass



import os
import shutil
import tarfile
import hashlib
from setuptools import Command
from glob import glob

try:
    from urllib2 import urlopen
except ImportError:
    from urllib.request import urlopen

from .msg import fatal, debug, info, warn

ENABLE_CHECKSUM = False

pjoin = os.path.join

# https://github.com/zeromq/czmq/archive/v4.0.2.tar.gz
bundled_version = (4, 1, 0)
vs = '%i.%i.%i' % bundled_version
libczmq = "czmq-%s.tar.gz" % vs
libczmq_url = "https://github.com/zeromq/czmq/archive/v{vs}.tar.gz".format(
    vs=vs,
)
libczmq_checksum = "sha256:2e87c19988d1168b70d7ec0fdce79aba4e92a6330959c3a2576c72b319acb478"

if os.getenv("PYZYRE_BUILD_MASTER", False) == '1':
    libczmq_url = "https://github.com/zeromq/czmq/archive/master.tar.gz"


HERE = os.path.dirname(__file__)
ROOT = os.path.dirname(HERE)


# -----------------------------------------------------------------------------
# Utilities
# -----------------------------------------------------------------------------

def untgz(archive):
    return archive.replace('.tar.gz', '')


def localpath(*args):
    """construct an absolute path from a list relative to the root pyzmq directory"""
    plist = [ROOT] + list(args)
    return os.path.abspath(pjoin(*plist))


def checksum_file(scheme, path):
    """Return the checksum (hex digest) of a file"""
    h = getattr(hashlib, scheme)()

    with open(path, 'rb') as f:
        chunk = f.read(65535)
        while chunk:
            h.update(chunk)
            chunk = f.read(65535)
    return h.hexdigest()


def fetch_archive(savedir, url, fname, checksum, force=False):
    """download an archive to a specific location"""
    dest = pjoin(savedir, fname)
    scheme, digest_ref = checksum.split(':')

    if os.path.exists(dest) and not force:
        info("already have %s" % dest)
        digest = checksum_file(scheme, fname)
        if libczmq_url == 'https://github.com/zeromq/czmq/archive/master.tar.gz':
            return dest
        if digest == digest_ref:
            return dest
        else:
            warn("but checksum %s != %s, redownloading." % (digest, digest_ref))
            os.remove(fname)

    info("fetching %s into %s" % (url, savedir))
    if not os.path.exists(savedir):
        os.makedirs(savedir)
    req = urlopen(url)
    with open(dest, 'wb') as f:
        f.write(req.read())
    digest = checksum_file(scheme, dest)
    if not ENABLE_CHECKSUM:
        return dest
    if digest != digest_ref and libczmq_url != 'https://github.com/zeromq/czmq/archive/master.tar.gz':
        fatal("%s %s mismatch:\nExpected: %s\nActual  : %s" % (
            dest, scheme, digest_ref, digest))
    return dest


def fetch_libczmq(savedir):
    dest = pjoin(savedir, 'czmq')
    if os.path.exists(dest):
        info("already have %s" % dest)
        return
    path = fetch_archive(savedir, libczmq_url, fname=libczmq, checksum=libczmq_checksum)
    tf = tarfile.open(path)
    with_version = pjoin(savedir, tf.firstmember.path)
    tf.extractall(savedir)
    tf.close()
    # remove version suffix:
    shutil.move(with_version, dest)


class FetchCommand(Command):

    description = "Fetch libczmq sources into bundled/czmq"

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        # fetch sources for libzmq extension:
        bundledir = "bundled"
        if os.path.exists(bundledir):
            info("Scrubbing directory: %s" % bundledir)
            shutil.rmtree(bundledir)
        if not os.path.exists(bundledir):
            os.makedirs(bundledir)
        #fetch_libzmq(bundledir)
        fetch_libczmq(bundledir)
        for tarball in glob(pjoin(bundledir, '*.tar.gz')):
            os.remove(tarball)



# this is really only to maintain versions, you should use and have access to czmq.* and zyre.*

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions

from zyre import *


import os
import re
import socket
import netifaces as ni
import netaddr
import dns.resolver
from pprint import pprint


def resolve_address(i):
    return interface_to_address(i)


def interface_to_address(i):
    if ':*' in i:
        i = i.split(':')[0]

    i = ni.ifaddresses(i)
    i = i[ni.AF_INET]
    i = i[0]['addr']
    return i


def default_address():
    i = default_interface()
    return interface_to_address(i)


def default_interface():
    try:
        return ni.gateways()['default'][ni.gateways()['default'].keys()[0]][1]
    except IndexError:
        raise RuntimeError("Unable to determine endpoint address")


def resolve_endpoint(endpoint, port):
    if len(endpoint) <= 5:
        # we likely have an interface
        if endpoint == '*':
            endpoint = default_address()
            endpoint = 'tcp://{}:{}'.format(endpoint, port)
            return endpoint

        endpoint = interface_to_address(endpoint)
        endpoint = 'tcp://{}:{}'.format(endpoint, port)
        return endpoint

    if endpoint.startswith('ipc://'):
        return endpoint

    if os.path.exists(endpoint):
        return 'ipc://{}'.format(endpoint)

    if endpoint.startswith(('tcp://', 'udp://')):
        if not re.search(r':\d{1,5}$', endpoint):
            endpoint = '%s:%s' % (endpoint, port)

        return endpoint

    # is it an ip address?
    try:
        socket.inet_aton(endpoint)
    except socket.error:
        return None

    # is it an FQDN?
    try:
        socket.gethostbyname(endpoint)
    except socket.error:
        return None

    if 'tcp://' not in endpoint:
        endpoint = 'tcp://%s' % endpoint

    if not re.search(r':\d{1,5}$', endpoint):
        endpoint = '%s:%s' % (endpoint, port)

    return endpoint


def address_to_interface(address):
    address = netaddr.IPAddress(address)

    for i in ni.interfaces():
        ii = ni.ifaddresses(i)
        if not ii.get(ni.AF_INET):
            continue

        ii = ii[ni.AF_INET]
        ii = netaddr.IPNetwork('{}/{}'.format(ii[0]['addr'], ii[0]['netmask']))
        try:
            if address in ii:
                return i
        except AttributeError:
            pass



# http://stackoverflow.com/a/1336640
# http://creativecommons.org/licenses/by-sa/3.0/

import logging


# now we patch Python code to add color support to logging.StreamHandler
def add_coloring_to_emit_windows(fn):
    # add methods we need to the class
    def _out_handle(self):
        import ctypes
        return ctypes.windll.kernel32.GetStdHandle(self.STD_OUTPUT_HANDLE)

    out_handle = property(_out_handle)

    def _set_color(self, code):
        import ctypes
        # Constants from the Windows API
        self.STD_OUTPUT_HANDLE = -11
        hdl = ctypes.windll.kernel32.GetStdHandle(self.STD_OUTPUT_HANDLE)
        ctypes.windll.kernel32.SetConsoleTextAttribute(hdl, code)

    setattr(logging.StreamHandler, '_set_color', _set_color)

    def new(*args):
        FOREGROUND_BLUE = 0x0001  # text color contains blue.
        FOREGROUND_GREEN = 0x0002  # text color contains green.
        FOREGROUND_RED = 0x0004  # text color contains red.
        FOREGROUND_INTENSITY = 0x0008  # text color is intensified.
        FOREGROUND_WHITE = FOREGROUND_BLUE | FOREGROUND_GREEN | FOREGROUND_RED
        # winbase.h
        STD_INPUT_HANDLE = -10
        STD_OUTPUT_HANDLE = -11
        STD_ERROR_HANDLE = -12

        # wincon.h
        FOREGROUND_BLACK = 0x0000
        FOREGROUND_BLUE = 0x0001
        FOREGROUND_GREEN = 0x0002
        FOREGROUND_CYAN = 0x0003
        FOREGROUND_RED = 0x0004
        FOREGROUND_MAGENTA = 0x0005
        FOREGROUND_YELLOW = 0x0006
        FOREGROUND_GREY = 0x0007
        FOREGROUND_INTENSITY = 0x0008  # foreground color is intensified.

        BACKGROUND_BLACK = 0x0000
        BACKGROUND_BLUE = 0x0010
        BACKGROUND_GREEN = 0x0020
        BACKGROUND_CYAN = 0x0030
        BACKGROUND_RED = 0x0040
        BACKGROUND_MAGENTA = 0x0050
        BACKGROUND_YELLOW = 0x0060
        BACKGROUND_GREY = 0x0070
        BACKGROUND_INTENSITY = 0x0080  # background color is intensified.

        levelno = args[1].levelno
        if (levelno >= 50):
            color = BACKGROUND_YELLOW | FOREGROUND_RED | FOREGROUND_INTENSITY | BACKGROUND_INTENSITY
        elif (levelno >= 40):
            color = FOREGROUND_RED | FOREGROUND_INTENSITY
        elif (levelno >= 30):
            color = FOREGROUND_YELLOW | FOREGROUND_INTENSITY
        elif (levelno >= 20):
            color = FOREGROUND_GREEN
        elif (levelno >= 10):
            color = FOREGROUND_MAGENTA
        else:
            color = FOREGROUND_WHITE
        args[0]._set_color(color)

        ret = fn(*args)
        args[0]._set_color(FOREGROUND_WHITE)
        # print "after"
        return ret

    return new


def add_coloring_to_emit_ansi(fn):
    # add methods we need to the class
    def new(*args):
        levelno = args[1].levelno
        if (levelno >= 50):
            color = '\x1b[31m'  # red
        elif (levelno >= 40):
            color = '\x1b[31m'  # red
        elif (levelno >= 30):
            color = '\x1b[33m'  # yellow
        elif (levelno >= 20):
            color = '\x1b[32m'  # green
        elif (levelno >= 10):
            color = '\x1b[35m'  # pink
        else:
            color = '\x1b[0m'  # normal

        if not args[1].msg:
            color = '\x1b[31m'
            args[1].msg = 'NoneType'

        args[1].msg = color + str(args[1].msg) + '\x1b[0m'  # normal
        # print "after"
        return fn(*args)

    return new


import platform

if platform.system() == 'Windows':
    # Windows does not support ANSI escapes and we are using API calls to set the console color
    logging.StreamHandler.emit = add_coloring_to_emit_windows(logging.StreamHandler.emit)
else:
    # all non-Windows platforms are supporting ANSI escapes so we use them
    logging.StreamHandler.emit = add_coloring_to_emit_ansi(logging.StreamHandler.emit)
    # log = logging.getLogger()
    # log.addFilter(log_filter())
    # //hdlr = logging.StreamHandler()
    # //hdlr.setFormatter(formatter())



import netifaces as ni
import sys
from time import sleep

import pytest

import zmq
import zmq.auth
from czmq import Zcert
from pyzyre.client import Client
from pyzyre.client._task import task
from zmq.auth.thread import ThreadAuthenticator
from zmq.eventloop import ioloop

ioloop.install()

@pytest.fixture
def iface():
    if sys.platform == 'darwin':
        return 'en0'
    else:
        if 'eth0' in ni.interfaces():
            return 'eth0'
        else:
            return ni.interfaces()[1]


def test_client_beacon(iface):
    # test with multi-process framework..
    c1 = Client(interface=iface)
    c1.start_zyre()

    c2 = Client(interface=iface)
    c2.start_zyre()

    sleep(0.01)

    loop = ioloop.IOLoop.instance()
    loop.add_handler(c1.actor, c1.handle_message, zmq.POLLIN)

    def test_fcn():
        c2.shout('ZYRE', 'TEST')
        sleep(1)

    loop.run_sync(test_fcn)

    c1.stop_zyre()
    c2.stop_zyre()

    loop.remove_handler(c1.actor)

    # cleanup
    sleep(2)


def test_client_beacon_curve(iface):
    #zauth = Client()._zauth()
    zauth = None

    cert1 = Zcert()
    assert(cert1.public_txt())

    c1 = Client(interace=iface, cert=cert1, zauth=zauth)
    c1.start_zyre()

    sleep(0.01)

    cert2 = Zcert()
    assert(cert2.public_txt())

    c2 = Client(interface=iface, task=task, verbose='1', cert=cert2, zauth=zauth)
    c2.start_zyre()

    sleep(0.01)

    loop = ioloop.IOLoop.instance()
    loop.add_handler(c1.actor, c1.handle_message, zmq.POLLIN)

    def test_fcn():
        c2.shout('ZYRE', 'TEST')
        sleep(1)

    loop.run_sync(test_fcn)

    loop.remove_handler(c1.actor)

    c1.stop_zyre()
    c2.stop_zyre()

    # cleanup
    sleep(2)


def test_client_gossip(iface):
    # test with multi-process framework..
    c1 = Client(gossip_bind='ipc:///tmp/gossip.ipc', endpoint='ipc:///tmp/c1.ipc')
    c1.start_zyre()

    c2 = Client(gossip_connect='ipc:///tmp/gossip.ipc', endpoint='ipc:///tmp/c2.ipc')
    c2.start_zyre()

    sleep(0.01)

    loop = ioloop.IOLoop.instance()
    loop.add_handler(c1.actor, c1.handle_message, zmq.POLLIN)

    def test_fcn():
        c2.shout('ZYRE', 'TEST')
        sleep(1)

    loop.run_sync(test_fcn)

    c1.stop_zyre()
    c2.stop_zyre()

    loop.remove_handler(c1.actor)

    # cleanup
    sleep(1)


def test_client_gossip_curve(iface):
    ctx = zmq.Context.instance()
    auth = ThreadAuthenticator(ctx)
    auth.start()

    # Tell authenticator to use the certificate in a directory
    auth.configure_curve(domain='*', location=zmq.auth.CURVE_ALLOW_ANY)

    cert1 = Zcert()
    assert (cert1.public_txt())

    gossip_cert = Zcert()
    assert (gossip_cert.public_txt())

    # test with multi-process framework..
    c1 = Client(gossip_bind='ipc:///tmp/gossip.ipc', endpoint='ipc:///tmp/c1.ipc', cert=cert1)
    c1.start_zyre()

    cert2 = Zcert()
    assert (cert2.public_txt())

    c2 = Client(gossip_connect='ipc:///tmp/gossip.ipc', endpoint='ipc:///tmp/c2.ipc',
                cert=cert2, gossip_publickey=cert1.public_txt())
    c2.start_zyre()

    sleep(0.01)

    loop = ioloop.IOLoop.instance()
    loop.add_handler(c1.actor, c1.handle_message, zmq.POLLIN)

    def test_fcn():
        c2.shout('ZYRE', 'TEST')
        sleep(1)

    loop.run_sync(test_fcn)

    c1.stop_zyre()
    c2.stop_zyre()

    loop.remove_handler(c1.actor)

    # cleanup
    sleep(1)


from __future__ import with_statement, print_function

from setuptools import Command
import os
from os.path import join as pjoin
from glob import glob
import shutil
import sys

class CleanCommand(Command):
    """Custom distutils command to clean the .so and .pyc files."""
    user_options = [('all', 'a',
                     "remove all build output, not just temporary by-products")
                    ]

    boolean_options = ['all']

    def initialize_options(self):
        self.all = None

    def finalize_options(self):
        pass

    def run(self):
        _clean_me = []
        _clean_trees = []

        for d in ('build', 'dist', 'conf'):
            if os.path.exists(d):
                _clean_trees.append(d)

        for root, dirs, files in os.walk('buildutils'):
            if any(root.startswith(pre) for pre in _clean_trees):
                continue
            for f in files:
                if os.path.splitext(f)[-1] == '.pyc':
                    _clean_me.append(pjoin(root, f))

            if '__pycache__' in dirs:
                _clean_trees.append(pjoin(root, '__pycache__'))

        for root, dirs, files in os.walk('zmq'):
            if any(root.startswith(pre) for pre in _clean_trees):
                continue

            for f in files:
                if os.path.splitext(f)[-1] in ('.pyc', '.so', '.o', '.pyd', '.json'):
                    _clean_me.append(pjoin(root, f))

            for d in dirs:
                if d == '__pycache__':
                    _clean_trees.append(pjoin(root, d))

        # remove generated cython files
        if self.all:
            for root, dirs, files in os.walk(pjoin('zmq', 'backend', 'cython')):
                if os.path.splitext(f)[-1] == '.c':
                    _clean_me.append(pjoin(root, f))

        bundled = glob(pjoin('zmq', 'libzmq*'))
        _clean_me.extend([b for b in bundled if b not in _clean_me])

        bundled_headers = glob(pjoin('zmq', 'include', '*.h'))
        _clean_me.extend([h for h in bundled_headers if h not in _clean_me])

        for clean_me in _clean_me:
            print("removing %s" % clean_me)
            try:
                os.unlink(clean_me)
            except Exception as e:
                print(e, file=sys.stderr)
                for clean_tree in _clean_trees:
                    print("removing %s/" % clean_tree)
                    try:
                        shutil.rmtree(clean_tree)
                    except Exception as e:
                        print(e, file=sys.stderr)



import logging
import os
import os.path
from time import sleep
import names
from pprint import pprint
import re

import zmq
from czmq import Zactor, zactor_fn, create_string_buffer, lib

from ._task import task as client_task
from ..utils import resolve_endpoint
from pyzyre.constants import GOSSIP_PORT, SERVICE_PORT, ZYRE_GROUP, GOSSIP_CONNECT, ENDPOINT, CURVE_ALLOW_ANY, \
    NODE_NAME, DefaultHandler

ZAUTH_TRACE = os.getenv('ZAUTH_TRACE', False)

logger = logging.getLogger(__name__)


class Client(object):

    def __init__(self, handler=DefaultHandler(), **kwargs):

        # disable CZMQ from capturing SIGINT
        os.environ['ZSYS_SIGHANDLER'] = 'false'

        self.handler = handler

        self.group = kwargs.get('group', ZYRE_GROUP)
        self._groups = self.group.split(',')
        self.group = '|'.join(self.group.split(','))
        self.interface = kwargs.get('interface') or '*'

        self.parent_loop = kwargs.get('loop')
        self.gossip_bind = kwargs.get('gossip_bind')
        self.beacon = kwargs.get('beacon')
        self.gossip_connect = kwargs.get('gossip_connect', GOSSIP_CONNECT)
        self.endpoint = kwargs.get('endpoint', ENDPOINT)
        self.cert = kwargs.get('cert')
        self.gossip_publickey = kwargs.get('gossip_publickey')
        self.first_node = None
        self.zauth = kwargs.get('zauth_curve_allow')
        self.advertised_endpoint = kwargs.get('advertised_endpoint')

        self.name = kwargs.get('name', NODE_NAME)
        if not self.name:
            self.name = '{}_{}'.format(names.get_first_name().lower(), names.get_last_name().lower())

        self.actor = None
        self.task = zactor_fn(client_task)

        if self.gossip_bind:
            self.beacon = False
            self.gossip_connect = False

        elif self.gossip_connect:
            self.beacon = False

        else:
            self.endpoint = False
            self.beacon = True

        if isinstance(self.zauth, str):
            self.zauth = self._zauth(self.zauth)

        self._init_zyre()

    def _zauth(self, allow=CURVE_ALLOW_ANY):
        logger.debug("spinning up zauth..")

        zauth = Zactor(zactor_fn(lib.zauth), None)

        if ZAUTH_TRACE:
            logger.debug('turning on auth verbose')
            lib.zstr_sendx(zauth, "VERBOSE", None)
            zauth.sock().wait()

        if allow:
            logger.debug("configuring Zauth...")
            lib.zstr_sendx(zauth, "CURVE", allow, None)
            zauth.sock().wait()

        logger.debug('Zauth complete')

        return zauth

    def _init_gossip_connect(self):
        try:
            logger.debug('resolving gossip-connect: {}'.format(self.gossip_connect))
            self.gossip_connect = resolve_endpoint(self.gossip_connect, GOSSIP_PORT)

        except RuntimeError as e:
            logger.error(e)
            logger.debug('falling back to beacon mode..')
            self.beacon = 1
            self.gossip_connect = None

        self.endpoint = resolve_endpoint(self.endpoint, SERVICE_PORT)

    def _init_gossip_bind(self):
        self.gossip_bind = resolve_endpoint(self.gossip_bind, GOSSIP_PORT)
        self.endpoint = resolve_endpoint(self.endpoint, SERVICE_PORT)

    def _init_zyre(self):
        # setup czmq/zyre
        # disable CZMQ from capturing SIGINT
        os.environ['ZSYS_SIGHANDLER'] = 'false'

        if self.gossip_bind:
            self._init_gossip_bind()

        if self.gossip_connect:
            self._init_gossip_connect()

        if self.beacon:
            os.environ["ZSYS_INTERFACE"] = self.interface

        actor_args = [
            'group=%s' % self.group,
            'name=%s' % self.name,
        ]

        if self.gossip_bind:
            actor_args.append('gossip_bind=%s' % self.gossip_bind)

        elif self.gossip_connect:
            actor_args.append('gossip_connect=%s' % self.gossip_connect)

        else:
            actor_args.append('beacon=1')

        if self.endpoint:
            actor_args.append('endpoint=%s' % self.endpoint)

        if self.advertised_endpoint:
            actor_args.append('advertised_endpoint=%s' % self.advertised_endpoint)

        if self.cert:
            actor_args.append('publickey=%s' % self.cert.public_txt())
            actor_args.append('secretkey=%s' % self.cert.secret_txt())

        if self.gossip_publickey:
            actor_args.append('gossip_publickey=%s' % self.gossip_publickey)

        actor_args = ','.join(actor_args)
        self.actor_args = create_string_buffer(actor_args)

        self.actor = None
        self._actor = None

    def start_zyre(self):
        self._actor = Zactor(self.task, self.actor_args)
        self.actor = zmq.Socket(shadow=self._actor.resolve(self._actor).value)

    def stop_zyre(self):
        self.actor.send_multipart(['$$STOP'.encode('utf-8')])
        m = self.actor.recv_multipart()
        sleep(0.01)
        del self._actor

        if self.zauth:
            del self.zauth  # destroy old actor
            self.zauth = None  # re-establish attrib

    def join(self, group):
        logger.debug('sending join')
        self.actor.send_multipart(['JOIN', group.encode('utf-8')])

    def shout(self, group, message):
        if isinstance(message, str):
            message = message.decode('utf-8')
        self.actor.send_multipart(['SHOUT', group.encode('utf-8'), message.encode('utf-8')])

    def whisper(self, message, address):
        logger.debug('sending whisper to %s' % address)
        if isinstance(message, str):
            message = message.decode('utf-8')
        self.actor.send_multipart(['WHISPER', address, message.encode('utf-8')])
        logger.debug('message sent via whisper')

    def leave(self, group):
        logger.debug('sending LEAVE for %s' % group)
        self.actor.send_multipart(['LEAVE', group])

    def handle_message(self, s, e):
        m = s.recv_multipart()

        m_type = m.pop(0)

        if m_type == 'SHOUT':
            group, peer, address, message = m
            self.handler.on_shout(self, group, peer, address, message)

        elif m_type == 'ENTER':
            # set teh first node name in case we need it later (gossip)
            if not self.first_node and self.gossip_connect:
                self.first_node = m[1]

            self.handler.on_enter(self, peer=m)

        elif m_type == 'WHISPER':
            peer, message = m
            self.handler.on_whisper(self, peer, message)

        elif m_type == 'EXIT':
            peer, peers_remining = m
            self.handler.on_exit(self, peer)

        elif m_type == 'JOIN':
            peer, group = m
            self.handler.on_join(self, peer, group)

        elif m_type == 'LEAVE':
            peer, group = m
            self.handler.on_leave(self, peer, group)

        elif m_type == 'EVASIVE':
            peer = m
            self.handler.on_evasive(self, peer)

        else:
            logger.warn("unhandled m_type {} rest of message is {}".format(m_type, m))





import logging
import textwrap
from argparse import ArgumentParser, RawDescriptionHelpFormatter
import os

import zmq
from .client import Client, DefaultHandler
from zmq.eventloop import ioloop
from .utils import get_argument_parser, setup_logging, setup_curve
from .constants import NODE_NAME
from pprint import pprint

CERT_PATH = os.getenv('ZYRE_CERT_PATH', os.path.expanduser('~/.curve'))
SECRET_KEY_PATH = os.getenv('ZYRE_CERT_PATH', os.path.expanduser('~/.curve/broker.key_secret'))

logger = logging.getLogger(__name__)


class BrokerHandler(DefaultHandler):
    def on_shout(self, client, group, peer, address, message):
        logger.info('SHOUT[{}][{}]: {}'.format(group, peer, message))


# see examples/pub.py and sub.py
def main():
    p = get_argument_parser()
    p = ArgumentParser(
        description=textwrap.dedent('''\
                example usage:
                    $ zyre-broker -d
                    $ zyre-broker --curve
                '''),
        formatter_class=RawDescriptionHelpFormatter,
        prog='zyre-broker',
        parents=[p]
    )

    args = p.parse_args()

    setup_logging(args)

    if not NODE_NAME:
        args.name = 'broker'

    if os.path.exists(SECRET_KEY_PATH):
        if not args.cert:
            args.cert = SECRET_KEY_PATH

    args.cert = setup_curve(args)
    if args.cert:
        args.gossip_publickey = args.cert.public_txt()
        args.publickey = args.cert.public_txt()
        args.secretkey = args.cert.secret_txt()
        args.curve = True

    ioloop.install()
    loop = ioloop.IOLoop.instance()

    client = Client(
        handler=BrokerHandler(),
        loop=loop,
        **args.__dict__
    )

    client.start_zyre()
    loop.add_handler(client.actor, client.handle_message, zmq.POLLIN)

    terminated = False
    while not terminated:
        try:
            logger.info('starting loop...')
            loop.start()

        except KeyboardInterrupt:
            logger.info('SIGINT Received')
            terminated = True

        except Exception as e:
            terminated = True
            logger.error(e)

    logger.info('shutting down..')

    client.stop_zyre()


if __name__ == '__main__':
    main()
